"""add bundle_key column to artifacts

Revision ID: d38cca7e13b8
Revises: b37f05a2bfbf
Create Date: 2026-03-13 19:56:03.116922+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "d38cca7e13b8"
down_revision: Union[str, None] = "b37f05a2bfbf"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("artifacts", sa.Column("bundle_key", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("artifacts", "bundle_key")
    # ### end Alembic commands ###
