# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .branch_condition_criteria import BranchConditionCriteria


class BranchCondition(UniversalBaseModel):
    """
    Represents a single conditional branch edge within a ConditionalBlock.
    """

    id: typing.Optional[str] = None
    criteria: typing.Optional[BranchConditionCriteria] = None
    next_block_label: typing.Optional[str] = None
    description: typing.Optional[str] = None
    is_default: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
