"""add idx_workflow_scripts_org_script_id index

Revision ID: def7c03f425d
Revises: 7dc73e5b2fe4
Create Date: 2026-03-17 22:54:15.239784+00:00

"""

from typing import Sequence, Union

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "def7c03f425d"
down_revision: Union[str, None] = "7dc73e5b2fe4"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "idx_workflow_scripts_org_script_id", "workflow_scripts", ["organization_id", "script_id"], unique=False
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("idx_workflow_scripts_org_script_id", table_name="workflow_scripts")
    # ### end Alembic commands ###
