"""code_version db migration

Revision ID: 787109c06571
Revises: 01dbfbf87496
Create Date: 2026-03-25 01:06:29.590429+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "787109c06571"
down_revision: Union[str, None] = "01dbfbf87496"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflows", sa.Column("code_version", sa.Integer(), server_default=sa.text("2"), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflows", "code_version")
    # ### end Alembic commands ###
