"""user_context for credentials table

Revision ID: b37f05a2bfbf
Revises: 08f585f2e7a6
Create Date: 2026-03-13 01:22:48.294533+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "b37f05a2bfbf"
down_revision: Union[str, None] = "08f585f2e7a6"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("credentials", sa.Column("user_context", sa.String(length=1000), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("credentials", "user_context")
    # ### end Alembic commands ###
